/*
 * This is a little header file which doxygen parses to generate the main
 * documentation page
 */

/**
 * @mainpage AES Package
 *
 * @section description Description
 *
 *  The Advanced Encryption Standard (AES) algorithm is a block cipher algorithm that can be
 *  used to encrypt and decrypt information.  The AES algorithm provided operates on 128 bit
 *  blocks of input data, plain or cipher for encryption and decryption, respectively.  To perform
 *  the encryption and decryption the algorithm requires a cryptographic key.  The provided
 *  algorithm can take keys of size 128, 192, or 256 bits as an input.  The key is expanded into
 *  multiple different keys, called round keys.  The keys are then used a set of transformational
 *  steps are applied to the input data.  For decryption, the same key is expanded and a set of 
 *  reverse transformational steps are applied to the cipher text to produce the original plain text.
 *
 *  For more information on the AES algorithm please refer to the following sources: <BR>
 *  FIPS Advanced Encryption Standard - http://www.csrc.nist.gov/publications/fips/fips197/fips-197.pdf <BR>
 *
 * @subsection usecases Use Cases
 *
 *  The AES algorithm is a block cipher algorithm selected by NIST to be the U.S. government's
 *  designated encryption cipher.  AES was chosen to protect sensitive (unclassified) government,
 *  business, and financial institution information.  In addition, the U.S. government has ruled
 *  the algorithm can be used to protect its classified information up to SECRET and TOP SECRET
 *  levels.  However, TOP SECRET level information requires 192 or 256 bit key encryption.
 *  Currently, the AES algorithm is available by choice in many different security applications, 
 *  including SSL, and IPsec.  Some uses of the algorithm include encryption of data sent over
 *  an IP network and the encryption of software files.
 *
 * @subsection licenseinfo License, Standard, and Export Control Information
 *
 *  For the source code licensing, standard, and export control information please see the release notes
 *  provided with this package.
 *
 * @section libraries Target Library Information
 *
 *  Different libaries will be produced depending upon target build.  There are three library types.
 *  Libraries *_a.a* will contain assembly code.  Libraries *_c.a* will contain C source code which 
 *  for c64x and c64x+ targets may contain intrinsic optimized code.  Libraries *_cm.a*
 *  will contain the C model code of the assembly or intrinsic code contained in the *_a.a* or *_c.a* library.
 *  If a target build has a *_a.a* library it will always also have a *_cm.a* in case the component
 *  integrator would like to use the C model code instead of the assembly/intrinsic code.  For a
 *  guideline on which library combinations are required for linking please refer to linker 
 *  command files, *.cmd, located in the @c aes\\test\\lnkr\\\<target\>\\ directory.
 *
 * @subsection libnotes Library Notes
 *
 *  The following libraries are provided in this package:<BR>
 *
 *  -# aes_c.a64P - Little endian c64x+ COFF library containing c64x+ optimized AES code<BR>
 *  -# aes_cm.a64P - Little endian c64x+ COFF library containing C model AES code<BR>
 *  -# aes_c.a64Pe - Big endian c64x+ COFF library containing c64x+ optimized AES code<BR>
 *  -# aes_cm.a64Pe - Big endian c64x+ COFF library containing C model AES code<BR>
 *  -# aes_c.ae64P - Little endian c64x+ ELF library containing c64x+ optimized AES code<BR>
 *  -# aes_cm.ae64P - Little endian c64x+ ELF library containing C model AES code<BR>
 *  -# aes_c.ae64Pe - Big endian c64x+ ELF library containing c64x+ optimized AES code<BR>
 *  -# aes_cm.ae64Pe - Big endian c64x+ ELF library containing C model AES code<BR>
 *  -# aes_c.a55L - c55x library containing AES code<BR>
 *  -# aes_c.a11 - Little endian ARM11 library containing ARM11 optimized AES code<BR>
 *  -# aes_c.a11e - Big endian ARM11 library containing ARM11 optimized AES code<BR>
 *  -# aes_c.av7A - Little endian ARM7A library containing ARM7A optimized AES code<BR>
 *  -# aes_c.a66 - Little endian c66 COFF library containing c66 optimized AES code<BR>
 *  -# aes_cm.a66 - Little endian c66 COFF library containing C model AES code<BR>
 *  -# aes_c.a66e - Big endian c66 COFF library containing c66 optimized AES code<BR>
 *  -# aes_cm.a66e - Big endian c66 COFF library containing C model AES code<BR>
 *  -# aes_c.ae66 - Little endian c66 ELF library containing c66 optimized AES code<BR>
 *  -# aes_cm.ae66 - Little endian c66 ELF library containing C model AES code<BR>
 *  -# aes_c.ae66e - Big endian c66 ELF library containing c66 optimized AES code<BR>
 *  -# aes_cm.ae66e - Big endian c66 ELF library containing C model AES code<BR>
 *  -# aes_c.a674 - Little endian c674 COFF library containing c674 optimized AES code<BR>
 *  -# aes_cm.a674 - Little endian c674 COFF library containing C model AES code<BR>
 *  -# aes_c.a674e - Big endian c674 COFF library containing c674 optimized AES code<BR>
 *  -# aes_cm.a674e - Big endian c674 COFF library containing C model AES code<BR>
 *  -# aes_c.ae674 - Little endian c674 ELF library containing c674 optimized AES code<BR>
 *  -# aes_cm.ae674 - Little endian c674 ELF library containing C model AES code<BR>
 *  -# aes_c.ae674e - Big endian c674 ELF library containing c674 optimized AES code<BR>
 *  -# aes_cm.ae674e - Big endian c674 ELF library containing C model AES code<BR>
 
 *
 * @section integration Integration Information
 *  
 *  There is only one module in this package which is the AES module. 
 *  The API documentation is located at @ref AES. 
 *  The API header is located at @c aes\\aes.h. Target library 
 *  location is @c aes\\\<target\>\\aes_*.a*
 *  There may be two libraries (assembly and C) for release on some targets. For information on 
 *  which libraries are required for linking, refer to @c aes\\test\\lnkr\\\<target\>\\*.cmd files.
 *  Targets provided in this package may include one or more of c55l (55 large), 
 *  c64le (64 little endian), c64be (64 big endian), c64Ple (64+ little endian) and c64Pbe (64+ big endian),
 *  tiarmv6le (ARM11 little endian), tiarmv6be (ARM11 big endian).
 *  
 *  These names are standard xdc target names.
 *  For MIPS and Memory Consumption information, refer to documentation in AES's test package.
 *
 *  For consumers of this package that do not use xdc tools to build their sources (e.g use GNU make instead),
 *  xdc dependency will need to be emulated when building sources that include the API headers of this release.
 *  The following defines will need to be included when compiling sources:
 *  @verbatim 
      -Dti_targets_<target> -Dxdc_target_name__=<target> -Dxdc_target_types__=ti/targets/std.h -Dxdc_nolocalnames

    where <target> is one of C54, C55, C55_large, C64, C64_big_endian, C64P, C64P_big_endian, C66, C66_big_endian, 
    C674, C674_big_endian, C64P_elf, C64P_big_endian_elf, C66_elf, C66_big_endian_elf, C674_elf, C674_big_endian_elf,
    TIArmv6le or TIArmv6be which are xdc standard target names (note all targets may not apply to this release) 
    Here C64 means C64 little endian, C64P is c64x+ little endian (P is for + in c64x+) and C66 means C66 little
    endian
    @endverbatim
 *  Note @c std.h is needed from xdc tools package even if xdc is not used to build the sources.
 *
 * @section dependencies Dependent Packages
 *  - ti.mas.types (Common data type defintions)
 *  - ti.mas.util  (Common utilities)
 *  - ti.mas.secutil  (Common security utilities)
 *  - ti.mas.swtools (Internally used s/w tools)
 */

/**
 * @page Revision History
 * <center> <B> Revision History </B> </center>
 *
 * <TABLE align=center>
 * <TR>
 * <TD><B>Version</B></TD>
 * <TD><B>Comments</B></TD>
 * </TR>
 * <TR>
 * <TD>1.2.5.1</TD>
 * <TD>Fixed compiler option of allkey for GCArmv7A</TD>
 * </TR>
 * <TR>
 * <TD>1.2.5.0</TD>
 * <TD>Added GCArmv7A support</TD>
 * </TR>
 * <TR>
 * <TD>1.2.4.0</TD>
 * <TD>Use ti.mas.pktutl istead of ti.mas.util</TD>
 * </TR>
 * <TR>
 * <TD>1.2.3.0</TD>
 * <TD>Modified copyright text with BSD licensing information</TD>
 * </TR>
 * <TR>
 * <TD>1.2.2.0</TD>
 * <TD>Support for new tools, ELF support</TD>
 * </TR>
 * <TR>
 * <TD>1.2.1.0</TD>
 * <TD>Support for new tools, C66 support</TD>
 * </TR>
 * <TR>
 * <TD>1.1.0.0</TD>
 * <TD>Support for new tools, ARM11 support</TD>
 * </TR>
 * <TR>
 * <TD>1.0.0.2</TD>
 * <TD>Support for new tools, c55x support, bug fixes, package cleanup and licensing information added</TD>
 * </TR>
 * <TR>
 * <TD>1.0.0.1</TD>
 * <TD>Added little endian decrypt and improved cycle counts</TD>
 * </TR>
 * <TR>
 * <TD>1.0.0.0</TD>
 * <TD>Initial Version</TD>
 * </TR>
 * </TABLE>
 */

/**
 * @page Disclaimer
 *<center> <B>IMPORTANT NOTICE </B> </center>
 * Texas Instruments Incorporated and its subsidiaries (TI) reserve the right
 * to make corrections, modifications, enhancements, improvements, and other
 * changes to its products and services at any time and to discontinue any
 * product or service without notice. Customers should obtain the latest
 * relevant information before placing orders and should verify that such
 * information is current and complete. All products are sold subject to TIs
 * terms and conditions of sale supplied at the time of order acknowledgment.
 *
 * TI warrants performance of its hardware products to the specifications
 * applicable at the time of sale in accordance with TIs standard warranty.
 * Testing and other quality control techniques are used to the extent TI
 * deems necessary to support this warranty. Except where mandated by
 * government requirements, testing of all parameters of each product is not
 * necessarily performed.
 *
 * TI assumes no liability for applications assistance or customer product
 * design. Customers are responsible for their products and applications
 * using TI components. To minimize the risks associated with customer
 * products and applications, customers should provide adequate design and
 * operating safeguards.
 *
 * TI does not warrant or represent that any license, either express or
 * implied, is granted under any TI patent right, copyright, mask work right,
 * or other TI intellectual property right relating to any combination,
 * machine, or process in which TI products or services are used. Information
 * published by TI regarding third-party products or services does not
 * constitute a license from TI to use such products or services or a warranty
 * or endorsement thereof. Use of such information may require a license from a
 * third party under the patents or other intellectual property of the third
 * party, or a license from TI under the patents or other intellectual property
 * of TI.
 *
 * Reproduction of information in TI data books or data sheets is permissible
 * only if reproduction is without alteration and is accompanied by all
 * associated warranties, conditions, limitations, and notices. Reproduction
 * of this information with alteration is an unfair and deceptive business
 * practice. TI is not responsible or liable for such altered documentation.
 *
 * Resale of TI products or services with statements different from or beyond
 * the parameters stated by TI for that product or service voids all express
 * and any implied warranties for the associated TI product or service and is
 * an unfair and deceptive business practice. TI is not responsible or liable
 * for any such statements.
 *
 *
 *
 *
 * Mailing Address: <BR>
 * Texas Instruments <BR>
 * Post Office Box 655303 <BR>
 * Dallas, Texas 75265 <BR>
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * Information in this document is subject to change without notice.
 * Texas Instruments may have pending patent applications, trademarks,
 * copyrights, or other intellectual property rights covering matter in this
 * document.  The furnishing of this document is given for usage with Texas
 * Instruments products only and does not give you any license to the
 * intellectual property that might be contained within this document.
 * Texas Instruments makes no implied or expressed warranties in this document
 * and is not responsible for the products based from this document.
 *
 *
 */
