/******************************************************************************
 * FILE PURPOSE:  AES Decoder Key Expansion Routine 
 ******************************************************************************
 * FILE NAME:   aesKeyExpDec.c  
 *
 * DESCRIPTION: AES Decryption Key Expansion Algorithm Implementation 
 *
 *              Note: It runs on MIPS/ARM or DSP
 *              So maintain this support while you modify this code.
 *
 * The following source was taken from cryptographic software written by the below authors
 * and has been modified and adapted by TI
 *****************************************************************************
 * rijndael-alg-fst.c
 *
 * @version 3.0 (December 2000)
 *
 * Optimised ANSI C code for the Rijndael cipher (now AES)
 *
 * @author Vincent Rijmen <vincent.rijmen@esat.kuleuven.ac.be>
 * @author Antoon Bosselaers <antoon.bosselaers@esat.kuleuven.ac.be>
 * @author Paulo Barreto <paulo.barreto@terra.com.br>
 *
 * This code is hereby placed in the public domain.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
 
/* AES header files */
#include "aes.h"

extern const tulong aesTd0[];
extern const tulong aesTd1[];
extern const tulong aesTd2[];
extern const tulong aesTd3[];

extern const tulong aesTe4[];

/******************************************************************************
  * FUNCTION: aesKeyExpandDec    
  * PURPOSE: Expand the cipher key into the decryption key schedule
  ******************************************************************************/
tint aesKeyExpandDec(tulong *rk, tword cipherKey[], tint keyBits)
{
    tint Nr, i, j;
    tulong temp;      
    tulong *Td0 = (tulong *)&aesTd0;
    tulong *Td1 = (tulong *)&aesTd1;
    tulong *Td2 = (tulong *)&aesTd2;
    tulong *Td3 = (tulong *)&aesTd3;
    tulong *Te4 = (tulong *)&aesTe4;

  /*
    * Td0[x] = Si[x].[0e, 09, 0d, 0b];
    * Td1[x] = Si[x].[0b, 0e, 09, 0d];
    * Td2[x] = Si[x].[0d, 0b, 0e, 09];
    * Td3[x] = Si[x].[09, 0d, 0b, 0e];
    * Td4[x] = Si[x].[01, 01, 01, 01];
    */

    /* expand the cipher key: */
    Nr = aesKeyExpandEnc(rk, cipherKey, keyBits);
    /* invert the order of the round keys: */
    for (i = 0, j = 4*Nr; i < j; i += 4, j -= 4) {
        temp = rk[i    ]; rk[i    ] = rk[j    ]; rk[j    ] = temp;
        temp = rk[i + 1]; rk[i + 1] = rk[j + 1]; rk[j + 1] = temp;
        temp = rk[i + 2]; rk[i + 2] = rk[j + 2]; rk[j + 2] = temp;
        temp = rk[i + 3]; rk[i + 3] = rk[j + 3]; rk[j + 3] = temp;
    }
    /* apply the inverse MixColumn transform to all round keys but the first and the last: */
    for (i = 1; i < Nr; i++) {
        rk += 4;
        rk[0] =
            Td0[Te4[(rk[0] >> 24)       ] & 0xff] ^
            Td1[Te4[(rk[0] >> 16) & 0xff] & 0xff] ^
            Td2[Te4[(rk[0] >>  8) & 0xff] & 0xff] ^
            Td3[Te4[(rk[0]      ) & 0xff] & 0xff];
        rk[1] =
            Td0[Te4[(rk[1] >> 24)       ] & 0xff] ^
            Td1[Te4[(rk[1] >> 16) & 0xff] & 0xff] ^
            Td2[Te4[(rk[1] >>  8) & 0xff] & 0xff] ^
            Td3[Te4[(rk[1]      ) & 0xff] & 0xff];
        rk[2] =
            Td0[Te4[(rk[2] >> 24)       ] & 0xff] ^
            Td1[Te4[(rk[2] >> 16) & 0xff] & 0xff] ^
            Td2[Te4[(rk[2] >>  8) & 0xff] & 0xff] ^
            Td3[Te4[(rk[2]      ) & 0xff] & 0xff];
        rk[3] =
            Td0[Te4[(rk[3] >> 24)       ] & 0xff] ^
            Td1[Te4[(rk[3] >> 16) & 0xff] & 0xff] ^
            Td2[Te4[(rk[3] >>  8) & 0xff] & 0xff] ^
            Td3[Te4[(rk[3]      ) & 0xff] & 0xff];
    }
    return Nr;
}

/* nothing past this point */
