#ifndef _SHA1LOC_H
#define _SHA1LOC_H
/******************************************************************************
 * FILE PURPOSE:  SHA1 Message Authentication Algorithm Defines and Macros
 ******************************************************************************
 * FILE NAME:   sha1loc.h  
 *
 * DESCRIPTION: Secure Hash Algorithm 1 (SHA1) as per NIST FIPS PUB 180-1
 *
 *              Note: It runs on MIPS/ARM or DSP
 *              So maintain this support while you modify this code.
 *              
 * (C) Copyright 2008 Texas Instruments Inc - http://www.ti.com
 *
 * The following source was taken from cryptographic software written by Eric Young and has
 * been modified and adapted by TI
 *****************************************************************************
 * Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 *****************************************************************************/
 
#include <ti/mas/sha1/sha1.h>

#ifdef ti_targets_C55_large
#define _SHA1_C55_LARGE 1
#else
#define _SHA1_C55_LARGE 0
#endif

#ifdef ti_targets_C64P
#define _SHA1_C64P 1
#else
#define _SHA1_C64P 0
#endif

#ifdef ti_targets_elf_C64P
#define _SHA1_ELF_C64P 1
#else
#define _SHA1_ELF_C64P 0
#endif


#ifdef ti_targets_C64P_big_endian
#define _SHA1_C64P_BIG_ENDIAN 1
#else
#define _SHA1_C64P_BIG_ENDIAN 0
#endif

#ifdef ti_targets_elf_C64P_big_endian
#define _SHA1_ELF_C64P_BIG_ENDIAN 1
#else
#define _SHA1_ELF_C64P_BIG_ENDIAN 0
#endif


#ifdef ti_targets_C66
#define _SHA1_C66 1
#else
#define _SHA1_C66 0
#endif

#ifdef ti_targets_elf_C66
#define _SHA1_ELF_C66 1
#else
#define _SHA1_ELF_C66 0
#endif


#ifdef ti_targets_C66_big_endian
#define _SHA1_C66_BIG_ENDIAN 1
#else
#define _SHA1_C66_BIG_ENDIAN 0
#endif

#ifdef ti_targets_elf_C66_big_endian
#define _SHA1_ELF_C66_BIG_ENDIAN 1
#else
#define _SHA1_ELF_C66_BIG_ENDIAN 0
#endif

#ifdef ti_targets_C674
#define _SHA1_C674 1
#else
#define _SHA1_C674 0
#endif

#ifdef ti_targets_elf_C674
#define _SHA1_ELF_C674 1
#else
#define _SHA1_ELF_C674 0
#endif


#ifdef ti_targets_C674_big_endian
#define _SHA1_C674_BIG_ENDIAN 1
#else
#define _SHA1_C674_BIG_ENDIAN 0
#endif

#ifdef ti_targets_elf_C674_big_endian
#define _SHA1_ELF_C674_BIG_ENDIAN 1
#else
#define _SHA1_ELF_C674_BIG_ENDIAN 0
#endif




#ifdef ti_targets_arm_Arm11
#define _SHA1_ARM11 1
#else
#define _SHA1_ARM11 0
#endif


#ifdef ti_targets_arm_Arm11_big_endian
#define _SHA1_ARM11_BIG_ENDIAN 1
#else
#define _SHA1_ARM11_BIG_ENDIAN 0
#endif

#ifdef gnu_targets_arm_GCArmv7A
#define _SHA1_GCARMV7A 1
#else
#define _SHA1_GCARMV7A 0
#endif


/* Include target specific version of sha1 defines and macros. */
#if _SHA1_C55_LARGE
#include <ti/mas/sha1/src/c55/sha1port.h>
#elif _SHA1_C64P
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_C64P_BIG_ENDIAN
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ELF_C64P
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ELF_C64P_BIG_ENDIAN
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_C66
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_C66_BIG_ENDIAN
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ELF_C66
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ELF_C66_BIG_ENDIAN
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_C674
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_C674_BIG_ENDIAN
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ELF_C674
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ELF_C674_BIG_ENDIAN
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ARM11
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_ARM11_BIG_ENDIAN
#include <ti/mas/sha1/src/c64/sha1port.h>
#elif _SHA1_GCARMV7A
#include <ti/mas/sha1/src/arm/sha1port.h>
#else
#error invalid target
#endif

#define sha1_BLOCK_SIZE_IN_BYTE           64   /* Bytes */  

#define sha1_INIT_DATA_h0 0x67452301UL
#define sha1_INIT_DATA_h1 0xefcdab89UL
#define sha1_INIT_DATA_h2 0x98badcfeUL
#define sha1_INIT_DATA_h3 0x10325476UL
#define sha1_INIT_DATA_h4 0xc3d2e1f0UL

#endif /* _SHA1LOC_H */
/* nothing past this point */
